@@ -2,24 +2,24 @@
  * testcode/replay.c - store and use a replay of events for the DNS resolver.
  *
  * Copyright (c) 2007, NLnet Labs. All rights reserved.
- * 
+ *
  * This software is open source.
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 
+ *
  * Redistributions of source code must retain the above copyright notice,
  * this list of conditions and the following disclaimer.
- * 
+ *
  * Redistributions in binary form must reproduce the above copyright notice,
  * this list of conditions and the following disclaimer in the documentation
  * and/or other materials provided with the distribution.
- * 
+ *
  * Neither the name of the NLNET LABS nor the names of its contributors may
  * be used to endorse or promote products derived from this software without
  * specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -51,6 +51,7 @@
 #include "testcode/testpkts.h"
 #include "testcode/fake_event.h"
 #include "sldns/str2wire.h"
+#include "util/timeval_func.h"
 
 /** max length of lines in file */
 #define MAX_LINE_LEN 10240
@@ -59,35 +60,19 @@
  * Expand a macro
  * @param store: value storage
  * @param runtime: replay runtime for other stuff.
- * @param text: the macro text, after the ${, Updated to after the } when 
+ * @param text: the macro text, after the ${, Updated to after the } when
  * 	done (successfully).
  * @return expanded text, malloced. NULL on failure.
  */
-static char* macro_expand(rbtree_type* store, 
+static char* macro_expand(rbtree_type* store,
 	struct replay_runtime* runtime, char** text);
 
-/** compare of time values */
-static int
-timeval_smaller(const struct timeval* x, const struct timeval* y)
-{
-#ifndef S_SPLINT_S
-	if(x->tv_sec < y->tv_sec)
-		return 1;
-	else if(x->tv_sec == y->tv_sec) {
-		if(x->tv_usec <= y->tv_usec)
-			return 1;
-		else	return 0;
-	}
-	else	return 0;
-#endif
-}
-
-/** parse keyword in string. 
+/** parse keyword in string.
  * @param line: if found, the line is advanced to after the keyword.
  * @param keyword: string.
- * @return: true if found, false if not. 
+ * @return: true if found, false if not.
  */
-static int 
+static int
 parse_keyword(char** line, const char* keyword)
 {
 	size_t len = (size_t)strlen(keyword);
@@ -135,8 +120,8 @@ strip_end_white(char* p)
 	}
 }
 
-/** 
- * Read a range from file. 
+/**
+ * Read a range from file.
  * @param remain: Rest of line (after RANGE keyword).
  * @param in: file to read from.
  * @param name: name to print in errors.
@@ -181,7 +166,7 @@ replay_range_read(char* remain, FILE* in, const char* name,
 			strip_end_white(parse);
 			if(!extstrtoaddr(parse, &rng->addr, &rng->addrlen,
 				UNBOUND_DNS_PORT)) {
-				log_err("Line %d: could not read ADDRESS: %s", 
+				log_err("Line %d: could not read ADDRESS: %s",
 					pstate->lineno, parse);
 				free(rng);
 				return NULL;
@@ -255,8 +240,8 @@ read_assign_step(char* remain, struct replay_moment* mom)
 		fatal_exit("out of memory");
 }
 
-/** 
- * Read a replay moment 'STEP' from file. 
+/**
+ * Read a replay moment 'STEP' from file.
  * @param remain: Rest of line (after STEP keyword).
  * @param in: file to read from.
  * @param name: name to print in errors.
@@ -376,18 +361,18 @@ replay_moment_read(char* remain, FILE* in, const char* name,
 		strip_end_white(remain);
 		if(!extstrtoaddr(remain, &mom->addr, &mom->addrlen,
 			UNBOUND_DNS_PORT)) {
-			log_err("line %d: could not parse ADDRESS: %s", 
+			log_err("line %d: could not parse ADDRESS: %s",
 				pstate->lineno, remain);
 			free(mom);
 			return NULL;
 		}
-	} 
+	}
 	if(parse_keyword(&remain, "ELAPSE")) {
 		double sec;
 		errno = 0;
 		sec = strtod(remain, &remain);
 		if(sec == 0. && errno != 0) {
-			log_err("line %d: could not parse ELAPSE: %s (%s)", 
+			log_err("line %d: could not parse ELAPSE: %s (%s)",
 				pstate->lineno, remain, strerror(errno));
 			free(mom);
 			return NULL;
@@ -397,7 +382,7 @@ replay_moment_read(char* remain, FILE* in, const char* name,
 		mom->elapse.tv_usec = (int)((sec - (double)mom->elapse.tv_sec)
 			*1000000. + 0.5);
 #endif
-	} 
+	}
 
 	if(readentry) {
 		mom->match = read_entry(in, name, pstate, 1);
@@ -433,7 +418,7 @@ make_scenario(char* line)
 	return scen;
 }
 
-struct replay_scenario* 
+struct replay_scenario*
 replay_scenario_read(FILE* in, const char* name, int* lineno)
 {
 	char line[MAX_LINE_LEN];
@@ -451,7 +436,7 @@ replay_scenario_read(FILE* in, const char* name, int* lineno)
 		(*lineno)++;
 		while(isspace((unsigned char)*parse))
 			parse++;
-		if(!*parse) 
+		if(!*parse)
 			continue; /* empty line */
 		if(parse_keyword(&parse, ";"))
 			continue; /* comment */
@@ -462,11 +447,11 @@ replay_scenario_read(FILE* in, const char* name, int* lineno)
 			if(!scen)
 				fatal_exit("%d: could not make scen", *lineno);
 			continue;
-		} 
+		}
 		if(!scen)
 			fatal_exit("%d: expected SCENARIO", *lineno);
 		if(parse_keyword(&parse, "RANGE_BEGIN")) {
-			struct replay_range* newr = replay_range_read(parse, 
+			struct replay_range* newr = replay_range_read(parse,
 				in, name, &pstate, line);
 			if(!newr)
 				fatal_exit("%d: bad range", pstate.lineno);
@@ -474,12 +459,12 @@ replay_scenario_read(FILE* in, const char* name, int* lineno)
 			newr->next_range = scen->range_list;
 			scen->range_list = newr;
 		} else if(parse_keyword(&parse, "STEP")) {
-			struct replay_moment* mom = replay_moment_read(parse, 
+			struct replay_moment* mom = replay_moment_read(parse,
 				in, name, &pstate);
 			if(!mom)
 				fatal_exit("%d: bad moment", pstate.lineno);
 			*lineno = pstate.lineno;
-			if(scen->mom_last && 
+			if(scen->mom_last &&
 				scen->mom_last->time_step >= mom->time_step)
 				fatal_exit("%d: time goes backwards", *lineno);
 			if(scen->mom_last)
@@ -502,7 +487,7 @@ replay_scenario_read(FILE* in, const char* name, int* lineno)
 	return NULL;
 }
 
-void 
+void
 replay_scenario_delete(struct replay_scenario* scen)
 {
 	struct replay_moment* mom, *momn;
@@ -630,7 +615,7 @@ do_macro_recursion(rbtree_type* store, struct replay_runtime* runtime,
 {
 	char* after = at+2;
 	char* expand = macro_expand(store, runtime, &after);
-	if(!expand) 
+	if(!expand)
 		return NULL; /* expansion failed */
 	if(!do_buf_insert(at, remain, after, expand)) {
 		free(expand);
@@ -665,7 +650,7 @@ do_macro_variable(rbtree_type* store, char* buf, size_t remain)
 	}
 	/* terminator, we are working in macro_expand() buffer */
 	sv = *at;
-	*at = 0; 
+	*at = 0;
 	v = macro_getvar(store, name);
 	*at = sv;
 
@@ -816,7 +801,7 @@ macro_expand(rbtree_type* store, struct replay_runtime* runtime, char** text)
 		time_t res = 0;
 		if(runtime) {
 			struct fake_timer* t = first_timer(runtime);
-			if(t && (time_t)t->tv.tv_sec >= runtime->now_secs) 
+			if(t && (time_t)t->tv.tv_sec >= runtime->now_secs)
 				res = (time_t)t->tv.tv_sec - runtime->now_secs;
 		}
 		snprintf(buf, sizeof(buf), ARG_LL "d", (long long)res);
@@ -855,9 +840,9 @@ macro_expand(rbtree_type* store, struct replay_runtime* runtime, char** text)
 	if(dofunc) {
 		/* post process functions, buf has the argument(s) */
 		if(strncmp(buf, "ctime", 5) == 0) {
-			return do_macro_ctime(buf+6);	
+			return do_macro_ctime(buf+6);
 		} else if(strncmp(buf, "range", 5) == 0) {
-			return do_macro_range(buf+6);	
+			return do_macro_range(buf+6);
 		}
 	}
 	return strdup(buf);
@@ -891,7 +876,7 @@ macro_process(rbtree_type* store, struct replay_runtime* runtime, char* text)
 	return strdup(buf);
 }
 
-char* 
+char*
 macro_lookup(rbtree_type* store, char* name)
 {
 	struct replay_var* x = macro_getvar(store, name);
@@ -907,7 +892,7 @@ void macro_print_debug(rbtree_type* store)
 	}
 }
 
-int 
+int
 macro_assign(rbtree_type* store, char* name, char* value)
 {
 	struct replay_var* x = macro_getvar(store, name);
