@@ -339,64 +339,67 @@ extern int yydebug;
     VAR_CACHEDB_REDISEXPIRERECORDS = 540, /* VAR_CACHEDB_REDISEXPIRERECORDS  */
     VAR_CACHEDB_REDISPATH = 541,   /* VAR_CACHEDB_REDISPATH  */
     VAR_CACHEDB_REDISPASSWORD = 542, /* VAR_CACHEDB_REDISPASSWORD  */
-    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 543, /* VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM  */
-    VAR_FOR_UPSTREAM = 544,        /* VAR_FOR_UPSTREAM  */
-    VAR_AUTH_ZONE = 545,           /* VAR_AUTH_ZONE  */
-    VAR_ZONEFILE = 546,            /* VAR_ZONEFILE  */
-    VAR_MASTER = 547,              /* VAR_MASTER  */
-    VAR_URL = 548,                 /* VAR_URL  */
-    VAR_FOR_DOWNSTREAM = 549,      /* VAR_FOR_DOWNSTREAM  */
-    VAR_FALLBACK_ENABLED = 550,    /* VAR_FALLBACK_ENABLED  */
-    VAR_TLS_ADDITIONAL_PORT = 551, /* VAR_TLS_ADDITIONAL_PORT  */
-    VAR_LOW_RTT = 552,             /* VAR_LOW_RTT  */
-    VAR_LOW_RTT_PERMIL = 553,      /* VAR_LOW_RTT_PERMIL  */
-    VAR_FAST_SERVER_PERMIL = 554,  /* VAR_FAST_SERVER_PERMIL  */
-    VAR_FAST_SERVER_NUM = 555,     /* VAR_FAST_SERVER_NUM  */
-    VAR_ALLOW_NOTIFY = 556,        /* VAR_ALLOW_NOTIFY  */
-    VAR_TLS_WIN_CERT = 557,        /* VAR_TLS_WIN_CERT  */
-    VAR_TCP_CONNECTION_LIMIT = 558, /* VAR_TCP_CONNECTION_LIMIT  */
-    VAR_ANSWER_COOKIE = 559,       /* VAR_ANSWER_COOKIE  */
-    VAR_COOKIE_SECRET = 560,       /* VAR_COOKIE_SECRET  */
-    VAR_IP_RATELIMIT_COOKIE = 561, /* VAR_IP_RATELIMIT_COOKIE  */
-    VAR_FORWARD_NO_CACHE = 562,    /* VAR_FORWARD_NO_CACHE  */
-    VAR_STUB_NO_CACHE = 563,       /* VAR_STUB_NO_CACHE  */
-    VAR_LOG_SERVFAIL = 564,        /* VAR_LOG_SERVFAIL  */
-    VAR_DENY_ANY = 565,            /* VAR_DENY_ANY  */
-    VAR_UNKNOWN_SERVER_TIME_LIMIT = 566, /* VAR_UNKNOWN_SERVER_TIME_LIMIT  */
-    VAR_LOG_TAG_QUERYREPLY = 567,  /* VAR_LOG_TAG_QUERYREPLY  */
-    VAR_STREAM_WAIT_SIZE = 568,    /* VAR_STREAM_WAIT_SIZE  */
-    VAR_TLS_CIPHERS = 569,         /* VAR_TLS_CIPHERS  */
-    VAR_TLS_CIPHERSUITES = 570,    /* VAR_TLS_CIPHERSUITES  */
-    VAR_TLS_USE_SNI = 571,         /* VAR_TLS_USE_SNI  */
-    VAR_IPSET = 572,               /* VAR_IPSET  */
-    VAR_IPSET_NAME_V4 = 573,       /* VAR_IPSET_NAME_V4  */
-    VAR_IPSET_NAME_V6 = 574,       /* VAR_IPSET_NAME_V6  */
-    VAR_TLS_SESSION_TICKET_KEYS = 575, /* VAR_TLS_SESSION_TICKET_KEYS  */
-    VAR_RPZ = 576,                 /* VAR_RPZ  */
-    VAR_TAGS = 577,                /* VAR_TAGS  */
-    VAR_RPZ_ACTION_OVERRIDE = 578, /* VAR_RPZ_ACTION_OVERRIDE  */
-    VAR_RPZ_CNAME_OVERRIDE = 579,  /* VAR_RPZ_CNAME_OVERRIDE  */
-    VAR_RPZ_LOG = 580,             /* VAR_RPZ_LOG  */
-    VAR_RPZ_LOG_NAME = 581,        /* VAR_RPZ_LOG_NAME  */
-    VAR_DYNLIB = 582,              /* VAR_DYNLIB  */
-    VAR_DYNLIB_FILE = 583,         /* VAR_DYNLIB_FILE  */
-    VAR_EDNS_CLIENT_STRING = 584,  /* VAR_EDNS_CLIENT_STRING  */
-    VAR_EDNS_CLIENT_STRING_OPCODE = 585, /* VAR_EDNS_CLIENT_STRING_OPCODE  */
-    VAR_NSID = 586,                /* VAR_NSID  */
-    VAR_ZONEMD_PERMISSIVE_MODE = 587, /* VAR_ZONEMD_PERMISSIVE_MODE  */
-    VAR_ZONEMD_CHECK = 588,        /* VAR_ZONEMD_CHECK  */
-    VAR_ZONEMD_REJECT_ABSENCE = 589, /* VAR_ZONEMD_REJECT_ABSENCE  */
-    VAR_RPZ_SIGNAL_NXDOMAIN_RA = 590, /* VAR_RPZ_SIGNAL_NXDOMAIN_RA  */
-    VAR_INTERFACE_AUTOMATIC_PORTS = 591, /* VAR_INTERFACE_AUTOMATIC_PORTS  */
-    VAR_EDE = 592,                 /* VAR_EDE  */
-    VAR_INTERFACE_ACTION = 593,    /* VAR_INTERFACE_ACTION  */
-    VAR_INTERFACE_VIEW = 594,      /* VAR_INTERFACE_VIEW  */
-    VAR_INTERFACE_TAG = 595,       /* VAR_INTERFACE_TAG  */
-    VAR_INTERFACE_TAG_ACTION = 596, /* VAR_INTERFACE_TAG_ACTION  */
-    VAR_INTERFACE_TAG_DATA = 597,  /* VAR_INTERFACE_TAG_DATA  */
-    VAR_PROXY_PROTOCOL_PORT = 598, /* VAR_PROXY_PROTOCOL_PORT  */
-    VAR_STATISTICS_INHIBIT_ZERO = 599, /* VAR_STATISTICS_INHIBIT_ZERO  */
-    VAR_HARDEN_UNKNOWN_ADDITIONAL = 600 /* VAR_HARDEN_UNKNOWN_ADDITIONAL  */
+    VAR_CACHEDB_REDISLOGICALDB = 543, /* VAR_CACHEDB_REDISLOGICALDB  */
+    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 544, /* VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM  */
+    VAR_FOR_UPSTREAM = 545,        /* VAR_FOR_UPSTREAM  */
+    VAR_AUTH_ZONE = 546,           /* VAR_AUTH_ZONE  */
+    VAR_ZONEFILE = 547,            /* VAR_ZONEFILE  */
+    VAR_MASTER = 548,              /* VAR_MASTER  */
+    VAR_URL = 549,                 /* VAR_URL  */
+    VAR_FOR_DOWNSTREAM = 550,      /* VAR_FOR_DOWNSTREAM  */
+    VAR_FALLBACK_ENABLED = 551,    /* VAR_FALLBACK_ENABLED  */
+    VAR_TLS_ADDITIONAL_PORT = 552, /* VAR_TLS_ADDITIONAL_PORT  */
+    VAR_LOW_RTT = 553,             /* VAR_LOW_RTT  */
+    VAR_LOW_RTT_PERMIL = 554,      /* VAR_LOW_RTT_PERMIL  */
+    VAR_FAST_SERVER_PERMIL = 555,  /* VAR_FAST_SERVER_PERMIL  */
+    VAR_FAST_SERVER_NUM = 556,     /* VAR_FAST_SERVER_NUM  */
+    VAR_ALLOW_NOTIFY = 557,        /* VAR_ALLOW_NOTIFY  */
+    VAR_TLS_WIN_CERT = 558,        /* VAR_TLS_WIN_CERT  */
+    VAR_TCP_CONNECTION_LIMIT = 559, /* VAR_TCP_CONNECTION_LIMIT  */
+    VAR_ANSWER_COOKIE = 560,       /* VAR_ANSWER_COOKIE  */
+    VAR_COOKIE_SECRET = 561,       /* VAR_COOKIE_SECRET  */
+    VAR_IP_RATELIMIT_COOKIE = 562, /* VAR_IP_RATELIMIT_COOKIE  */
+    VAR_FORWARD_NO_CACHE = 563,    /* VAR_FORWARD_NO_CACHE  */
+    VAR_STUB_NO_CACHE = 564,       /* VAR_STUB_NO_CACHE  */
+    VAR_LOG_SERVFAIL = 565,        /* VAR_LOG_SERVFAIL  */
+    VAR_DENY_ANY = 566,            /* VAR_DENY_ANY  */
+    VAR_UNKNOWN_SERVER_TIME_LIMIT = 567, /* VAR_UNKNOWN_SERVER_TIME_LIMIT  */
+    VAR_LOG_TAG_QUERYREPLY = 568,  /* VAR_LOG_TAG_QUERYREPLY  */
+    VAR_STREAM_WAIT_SIZE = 569,    /* VAR_STREAM_WAIT_SIZE  */
+    VAR_TLS_CIPHERS = 570,         /* VAR_TLS_CIPHERS  */
+    VAR_TLS_CIPHERSUITES = 571,    /* VAR_TLS_CIPHERSUITES  */
+    VAR_TLS_USE_SNI = 572,         /* VAR_TLS_USE_SNI  */
+    VAR_IPSET = 573,               /* VAR_IPSET  */
+    VAR_IPSET_NAME_V4 = 574,       /* VAR_IPSET_NAME_V4  */
+    VAR_IPSET_NAME_V6 = 575,       /* VAR_IPSET_NAME_V6  */
+    VAR_TLS_SESSION_TICKET_KEYS = 576, /* VAR_TLS_SESSION_TICKET_KEYS  */
+    VAR_RPZ = 577,                 /* VAR_RPZ  */
+    VAR_TAGS = 578,                /* VAR_TAGS  */
+    VAR_RPZ_ACTION_OVERRIDE = 579, /* VAR_RPZ_ACTION_OVERRIDE  */
+    VAR_RPZ_CNAME_OVERRIDE = 580,  /* VAR_RPZ_CNAME_OVERRIDE  */
+    VAR_RPZ_LOG = 581,             /* VAR_RPZ_LOG  */
+    VAR_RPZ_LOG_NAME = 582,        /* VAR_RPZ_LOG_NAME  */
+    VAR_DYNLIB = 583,              /* VAR_DYNLIB  */
+    VAR_DYNLIB_FILE = 584,         /* VAR_DYNLIB_FILE  */
+    VAR_EDNS_CLIENT_STRING = 585,  /* VAR_EDNS_CLIENT_STRING  */
+    VAR_EDNS_CLIENT_STRING_OPCODE = 586, /* VAR_EDNS_CLIENT_STRING_OPCODE  */
+    VAR_NSID = 587,                /* VAR_NSID  */
+    VAR_ZONEMD_PERMISSIVE_MODE = 588, /* VAR_ZONEMD_PERMISSIVE_MODE  */
+    VAR_ZONEMD_CHECK = 589,        /* VAR_ZONEMD_CHECK  */
+    VAR_ZONEMD_REJECT_ABSENCE = 590, /* VAR_ZONEMD_REJECT_ABSENCE  */
+    VAR_RPZ_SIGNAL_NXDOMAIN_RA = 591, /* VAR_RPZ_SIGNAL_NXDOMAIN_RA  */
+    VAR_INTERFACE_AUTOMATIC_PORTS = 592, /* VAR_INTERFACE_AUTOMATIC_PORTS  */
+    VAR_EDE = 593,                 /* VAR_EDE  */
+    VAR_INTERFACE_ACTION = 594,    /* VAR_INTERFACE_ACTION  */
+    VAR_INTERFACE_VIEW = 595,      /* VAR_INTERFACE_VIEW  */
+    VAR_INTERFACE_TAG = 596,       /* VAR_INTERFACE_TAG  */
+    VAR_INTERFACE_TAG_ACTION = 597, /* VAR_INTERFACE_TAG_ACTION  */
+    VAR_INTERFACE_TAG_DATA = 598,  /* VAR_INTERFACE_TAG_DATA  */
+    VAR_PROXY_PROTOCOL_PORT = 599, /* VAR_PROXY_PROTOCOL_PORT  */
+    VAR_STATISTICS_INHIBIT_ZERO = 600, /* VAR_STATISTICS_INHIBIT_ZERO  */
+    VAR_HARDEN_UNKNOWN_ADDITIONAL = 601, /* VAR_HARDEN_UNKNOWN_ADDITIONAL  */
+    VAR_DISABLE_EDNS_DO = 602,     /* VAR_DISABLE_EDNS_DO  */
+    VAR_CACHEDB_NO_STORE = 603     /* VAR_CACHEDB_NO_STORE  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -690,64 +693,67 @@ extern int yydebug;
 #define VAR_CACHEDB_REDISEXPIRERECORDS 540
 #define VAR_CACHEDB_REDISPATH 541
 #define VAR_CACHEDB_REDISPASSWORD 542
-#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 543
-#define VAR_FOR_UPSTREAM 544
-#define VAR_AUTH_ZONE 545
-#define VAR_ZONEFILE 546
-#define VAR_MASTER 547
-#define VAR_URL 548
-#define VAR_FOR_DOWNSTREAM 549
-#define VAR_FALLBACK_ENABLED 550
-#define VAR_TLS_ADDITIONAL_PORT 551
-#define VAR_LOW_RTT 552
-#define VAR_LOW_RTT_PERMIL 553
-#define VAR_FAST_SERVER_PERMIL 554
-#define VAR_FAST_SERVER_NUM 555
-#define VAR_ALLOW_NOTIFY 556
-#define VAR_TLS_WIN_CERT 557
-#define VAR_TCP_CONNECTION_LIMIT 558
-#define VAR_ANSWER_COOKIE 559
-#define VAR_COOKIE_SECRET 560
-#define VAR_IP_RATELIMIT_COOKIE 561
-#define VAR_FORWARD_NO_CACHE 562
-#define VAR_STUB_NO_CACHE 563
-#define VAR_LOG_SERVFAIL 564
-#define VAR_DENY_ANY 565
-#define VAR_UNKNOWN_SERVER_TIME_LIMIT 566
-#define VAR_LOG_TAG_QUERYREPLY 567
-#define VAR_STREAM_WAIT_SIZE 568
-#define VAR_TLS_CIPHERS 569
-#define VAR_TLS_CIPHERSUITES 570
-#define VAR_TLS_USE_SNI 571
-#define VAR_IPSET 572
-#define VAR_IPSET_NAME_V4 573
-#define VAR_IPSET_NAME_V6 574
-#define VAR_TLS_SESSION_TICKET_KEYS 575
-#define VAR_RPZ 576
-#define VAR_TAGS 577
-#define VAR_RPZ_ACTION_OVERRIDE 578
-#define VAR_RPZ_CNAME_OVERRIDE 579
-#define VAR_RPZ_LOG 580
-#define VAR_RPZ_LOG_NAME 581
-#define VAR_DYNLIB 582
-#define VAR_DYNLIB_FILE 583
-#define VAR_EDNS_CLIENT_STRING 584
-#define VAR_EDNS_CLIENT_STRING_OPCODE 585
-#define VAR_NSID 586
-#define VAR_ZONEMD_PERMISSIVE_MODE 587
-#define VAR_ZONEMD_CHECK 588
-#define VAR_ZONEMD_REJECT_ABSENCE 589
-#define VAR_RPZ_SIGNAL_NXDOMAIN_RA 590
-#define VAR_INTERFACE_AUTOMATIC_PORTS 591
-#define VAR_EDE 592
-#define VAR_INTERFACE_ACTION 593
-#define VAR_INTERFACE_VIEW 594
-#define VAR_INTERFACE_TAG 595
-#define VAR_INTERFACE_TAG_ACTION 596
-#define VAR_INTERFACE_TAG_DATA 597
-#define VAR_PROXY_PROTOCOL_PORT 598
-#define VAR_STATISTICS_INHIBIT_ZERO 599
-#define VAR_HARDEN_UNKNOWN_ADDITIONAL 600
+#define VAR_CACHEDB_REDISLOGICALDB 543
+#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 544
+#define VAR_FOR_UPSTREAM 545
+#define VAR_AUTH_ZONE 546
+#define VAR_ZONEFILE 547
+#define VAR_MASTER 548
+#define VAR_URL 549
+#define VAR_FOR_DOWNSTREAM 550
+#define VAR_FALLBACK_ENABLED 551
+#define VAR_TLS_ADDITIONAL_PORT 552
+#define VAR_LOW_RTT 553
+#define VAR_LOW_RTT_PERMIL 554
+#define VAR_FAST_SERVER_PERMIL 555
+#define VAR_FAST_SERVER_NUM 556
+#define VAR_ALLOW_NOTIFY 557
+#define VAR_TLS_WIN_CERT 558
+#define VAR_TCP_CONNECTION_LIMIT 559
+#define VAR_ANSWER_COOKIE 560
+#define VAR_COOKIE_SECRET 561
+#define VAR_IP_RATELIMIT_COOKIE 562
+#define VAR_FORWARD_NO_CACHE 563
+#define VAR_STUB_NO_CACHE 564
+#define VAR_LOG_SERVFAIL 565
+#define VAR_DENY_ANY 566
+#define VAR_UNKNOWN_SERVER_TIME_LIMIT 567
+#define VAR_LOG_TAG_QUERYREPLY 568
+#define VAR_STREAM_WAIT_SIZE 569
+#define VAR_TLS_CIPHERS 570
+#define VAR_TLS_CIPHERSUITES 571
+#define VAR_TLS_USE_SNI 572
+#define VAR_IPSET 573
+#define VAR_IPSET_NAME_V4 574
+#define VAR_IPSET_NAME_V6 575
+#define VAR_TLS_SESSION_TICKET_KEYS 576
+#define VAR_RPZ 577
+#define VAR_TAGS 578
+#define VAR_RPZ_ACTION_OVERRIDE 579
+#define VAR_RPZ_CNAME_OVERRIDE 580
+#define VAR_RPZ_LOG 581
+#define VAR_RPZ_LOG_NAME 582
+#define VAR_DYNLIB 583
+#define VAR_DYNLIB_FILE 584
+#define VAR_EDNS_CLIENT_STRING 585
+#define VAR_EDNS_CLIENT_STRING_OPCODE 586
+#define VAR_NSID 587
+#define VAR_ZONEMD_PERMISSIVE_MODE 588
+#define VAR_ZONEMD_CHECK 589
+#define VAR_ZONEMD_REJECT_ABSENCE 590
+#define VAR_RPZ_SIGNAL_NXDOMAIN_RA 591
+#define VAR_INTERFACE_AUTOMATIC_PORTS 592
+#define VAR_EDE 593
+#define VAR_INTERFACE_ACTION 594
+#define VAR_INTERFACE_VIEW 595
+#define VAR_INTERFACE_TAG 596
+#define VAR_INTERFACE_TAG_ACTION 597
+#define VAR_INTERFACE_TAG_DATA 598
+#define VAR_PROXY_PROTOCOL_PORT 599
+#define VAR_STATISTICS_INHIBIT_ZERO 600
+#define VAR_HARDEN_UNKNOWN_ADDITIONAL 601
+#define VAR_DISABLE_EDNS_DO 602
+#define VAR_CACHEDB_NO_STORE 603
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -757,7 +763,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 761 "util/configparser.h"
+#line 767 "util/configparser.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
